@@ -1,6 +1,7 @@
 #' @title The Simple Two-Stage Estimation (TSE) Method for Treatment 
 #' Switching
-#' @description Obtains the causal parameter estimate of the AFT model 
+#' @description Obtains the causal parameter estimate of the accelerated 
+#' failure-time (AFT) model 
 #' for switching after disease progression and the hazard ratio estimate 
 #' of the outcome Cox model to adjust for treatment switching.
 #'
@@ -45,7 +46,7 @@
 #' @param swtrt_time The name of the swtrt_time variable in the input data.
 #' @param base_cov The names of baseline covariates (excluding
 #'   treat) in the input data for the outcome Cox model.
-#' @param base2_cov The names of secondary baseline covariates
+#' @param base2_cov The names of baseline and secondary baseline covariates
 #'   (excluding swtrt) in the input data for the AFT model for 
 #'   post-progression survival.
 #' @param aft_dist The assumed distribution for time to event for the AFT
@@ -102,10 +103,12 @@
 #'   i.e., "AFT model" or "bootstrap".
 #'   
 #' * \code{logrank_pvalue}: The two-sided p-value of the log-rank test
-#'   for an intention-to-treat (ITT) analysis.
+#'   for the ITT analysis.
 #'
 #' * \code{cox_pvalue}: The two-sided p-value for treatment effect based on
-#'   the Cox model.
+#'   the Cox model applied to counterfactual unswitched survival times.
+#'   If \code{boot} is \code{TRUE}, this value represents the 
+#'   bootstrap p-value.
 #'
 #' * \code{hr}: The estimated hazard ratio from the Cox model.
 #'
@@ -115,12 +118,17 @@
 #'   either "Cox model" or "bootstrap".
 #'
 #' * \code{data_aft}: A list of input data for the AFT model by treatment 
-#'   group.
+#'   group. The variables include \code{id}, \code{stratum}, \code{"pps"}, 
+#'   \code{"event"}, \code{"swtrt"}, \code{base2_cov}, \code{pd_time}, 
+#'   \code{swtrt_time}, and \code{time}.
 #' 
 #' * \code{fit_aft}: A list of fitted AFT models by treatment group.
 #' 
-#' * \code{data_outcome}: The input data for the outcome Cox model. 
-#' 
+#' * \code{data_outcome}: The input data for the outcome Cox model 
+#'   of counterfactual unswitched survival times.
+#'   The variables include \code{id}, \code{stratum}, \code{"t_star"}, 
+#'   \code{"d_star"}, \code{"treated"}, \code{base_cov}, and \code{treat}.
+#'
 #' * \code{fit_outcome}: The fitted outcome Cox model.
 #'
 #' * \code{settings}: A list with the following components:
@@ -304,5 +312,52 @@ tsesimp <- function(data, id = "id", stratum = "", time = "time",
   out$data_outcome$uid <- NULL
   out$data_outcome$ustratum <- NULL
   
+  
+  if (p >= 1) {
+    t1 = terms(formula(paste("~", paste(base_cov, collapse = "+"))))
+    t2 = attr(t1, "factors")
+    t3 = rownames(t2)
+    
+    add_vars <- setdiff(t3, varnames)
+    if (length(add_vars) > 0) {
+      out$data_outcome <- merge(out$data_outcome, df[, c(id, add_vars)], 
+                                by = id, all.x = TRUE, sort = FALSE)
+    }
+    
+    del_vars <- setdiff(varnames, t3)
+    if (length(del_vars) > 0) {
+      out$data_outcome[, del_vars] <- NULL
+    }
+  }
+  
+  if (p2 >= 1) {
+    t1 = terms(formula(paste("~", paste(base2_cov, collapse = "+"))))
+    t2 = attr(t1, "factors")
+    t3 = rownames(t2)
+    
+    K = ifelse(swtrt_control_only, 1, 2)
+    tem_vars <- c(pd_time, swtrt_time, time)
+    add_vars <- c(setdiff(t3, varnames2), tem_vars)
+    if (length(add_vars) > 0) {
+      for (h in 1:K) {
+        out$data_aft[[h]]$data <- merge(out$data_aft[[h]]$data, 
+                                        df[, c(id, add_vars)], 
+                                        by = id, all.x = TRUE, sort = FALSE)
+      }
+    }
+    
+    del_vars <- setdiff(varnames2, t3)
+    if (length(del_vars) > 0) {
+      for (h in 1:K) {
+        out$data_aft[[h]]$data[, del_vars] <- NULL
+      }
+    }
+    
+    for (h in 1:K) {
+      out$data_aft[[h]]$data <- out$data_aft[[h]]$data[
+        , !startsWith(names(out$data_aft[[h]]$data), "stratum_")]
+    }
+  }
+
   out
 }
\ No newline at end of file
