@@ -479,7 +479,7 @@ List tsesimpcpp(const DataFrame data,
                     IntegerVector swtrt2 = swtrtb[l];
                     
                     DataFrame data1 = DataFrame::create(
-                      Named("time") = time2,
+                      Named("pps") = time2,
                       Named("event") = event2,
                       Named("swtrt") = swtrt2);
                     
@@ -490,8 +490,9 @@ List tsesimpcpp(const DataFrame data,
                     }
                     
                     List fit1 = liferegcpp(
-                      data1, "", "", "time", "", "event", 
-                      covariates_aft, "", "", "", dist, 0, 0, alpha);
+                      data1, "", "", "pps", "", "event", 
+                      covariates_aft, "", "", "", dist, 0, 0, alpha, 
+                      50, 1.0e-9);
                     
                     DataFrame parest1 = DataFrame(fit1["parest"]);
                     NumericVector beta1 = parest1["beta"];
@@ -590,7 +591,8 @@ List tsesimpcpp(const DataFrame data,
                   
                   List fit_outcome = phregcpp(
                     data_outcome, "", "ustratum", "t_star", "", "d_star",
-                    covariates, "", "", "", ties, 0, 0, 0, 0, 0, alpha);
+                    covariates, "", "", "", ties, 0, 0, 0, 0, 0, alpha, 
+                    50, 1.0e-9);
                   
                   DataFrame parest = DataFrame(fit_outcome["parest"]);
                   NumericVector beta = parest["beta"];
