@@ -1,4 +1,5 @@
 library(dplyr, warn.conflicts = FALSE)
+library(survival)
 
 testthat::test_that("ipe: control to active switch", {
   data1 <- immdef %>% mutate(rx = 1-xoyrs/progyrs)
@@ -9,8 +10,9 @@ testthat::test_that("ipe: control to active switch", {
     boot = FALSE)
   
   # log-rank for ITT
-  fit_lr <- lrtest(data1, treat = "imm", time = "progyrs", event = "prog")
-  z_lr = fit_lr$logRankZ
+  fit_lr <- survdiff(Surv(progyrs, prog) ~ imm, data = data1)
+  z_lr <- sqrt(fit_lr$chisq)
+  if (fit_lr$obs[2] < fit_lr$exp[2]) z_lr <- -z_lr
   
   f <- function(psi) {
     data1 %>%
@@ -28,17 +30,17 @@ testthat::test_that("ipe: control to active switch", {
   
   g <- function(psi) {
     data2 <- f(psi)
-    fit_aft <- liferegr(data2, time = "t_star", event = "d_star", 
-                        covariates = "imm", dist = "weibull")
-    -fit_aft$beta[2] - psi
+    fit_aft <- survreg(Surv(t_star, d_star) ~ imm, 
+                       data = data2, dist = "weibull")
+    -fit_aft$coefficients[2] - psi
   }
   
   # psi based on AFT model
-  psi <- uniroot(g, c(-3,3), tol = 1e-6)$root
+  psi <- uniroot(g, c(-1,1), tol = 1e-6)$root
   
   data2 <- f(psi)
-  fit <- phregr(data2, time = "t_star", event = "d_star", covariates = "imm")
-  beta = as.numeric(fit$beta[1])
+  fit <- coxph(Surv(t_star, d_star) ~ imm, data = data2)
+  beta = as.numeric(fit$coefficients[1])
   se = beta/z_lr
   
   zcrit = qnorm(0.975)
